@@ -4,22 +4,23 @@ On Unix, this uses AF_UNIX sockets.
 On Windows, this uses NamedPipes.
 """
 
+from __future__ import annotations
+
 import base64
 import os
 import shutil
 import sys
 import tempfile
-
-from typing import Optional, Callable
-from typing_extensions import Final, Type
-
 from types import TracebackType
+from typing import Callable
+from typing_extensions import Final
 
-if sys.platform == 'win32':
+if sys.platform == "win32":
     # This may be private, but it is needed for IPC on Windows, and is basically stable
-    import _winapi
     import ctypes
 
+    import _winapi
+
     _IPCHandle = int
 
     kernel32 = ctypes.windll.kernel32
@@ -27,12 +28,12 @@ if sys.platform == 'win32':
     FlushFileBuffers: Callable[[_IPCHandle], int] = kernel32.FlushFileBuffers
 else:
     import socket
+
     _IPCHandle = socket.socket
 
 
 class IPCException(Exception):
     """Exception for IPC issues."""
-    pass
 
 
 class IPCBase:
@@ -44,14 +45,14 @@ class IPCBase:
 
     connection: _IPCHandle
 
-    def __init__(self, name: str, timeout: Optional[float]) -> None:
+    def __init__(self, name: str, timeout: float | None) -> None:
         self.name = name
         self.timeout = timeout
 
     def read(self, size: int = 100000) -> bytes:
         """Read bytes from an IPC connection until its empty."""
         bdata = bytearray()
-        if sys.platform == 'win32':
+        if sys.platform == "win32":
             while True:
                 ov, err = _winapi.ReadFile(self.connection, size, overlapped=True)
                 try:
@@ -85,7 +86,7 @@ class IPCBase:
 
     def write(self, data: bytes) -> None:
         """Write bytes to an IPC connection."""
-        if sys.platform == 'win32':
+        if sys.platform == "win32":
             try:
                 ov, err = _winapi.WriteFile(self.connection, data, overlapped=True)
                 # TODO: remove once typeshed supports Literal types
@@ -112,7 +113,7 @@ class IPCBase:
             self.connection.shutdown(socket.SHUT_WR)
 
     def close(self) -> None:
-        if sys.platform == 'win32':
+        if sys.platform == "win32":
             if self.connection != _winapi.NULL:
                 _winapi.CloseHandle(self.connection)
         else:
@@ -122,9 +123,9 @@ class IPCBase:
 class IPCClient(IPCBase):
     """The client side of an IPC connection."""
 
-    def __init__(self, name: str, timeout: Optional[float]) -> None:
+    def __init__(self, name: str, timeout: float | None) -> None:
         super().__init__(name, timeout)
-        if sys.platform == 'win32':
+        if sys.platform == "win32":
             timeout = int(self.timeout * 1000) if self.timeout else _winapi.NMPWAIT_WAIT_FOREVER
             try:
                 _winapi.WaitNamedPipe(self.name, timeout)
@@ -150,39 +151,41 @@ class IPCClient(IPCBase):
                     raise IPCException("The connection is busy.") from e
                 else:
                     raise
-            _winapi.SetNamedPipeHandleState(self.connection,
-                                            _winapi.PIPE_READMODE_MESSAGE,
-                                            None,
-                                            None)
+            _winapi.SetNamedPipeHandleState(
+                self.connection, _winapi.PIPE_READMODE_MESSAGE, None, None
+            )
         else:
             self.connection = socket.socket(socket.AF_UNIX)
             self.connection.settimeout(timeout)
             self.connection.connect(name)
 
-    def __enter__(self) -> 'IPCClient':
+    def __enter__(self) -> IPCClient:
         return self
 
-    def __exit__(self,
-                 exc_ty: 'Optional[Type[BaseException]]' = None,
-                 exc_val: Optional[BaseException] = None,
-                 exc_tb: Optional[TracebackType] = None,
-                 ) -> None:
+    def __exit__(
+        self,
+        exc_ty: type[BaseException] | None = None,
+        exc_val: BaseException | None = None,
+        exc_tb: TracebackType | None = None,
+    ) -> None:
         self.close()
 
 
 class IPCServer(IPCBase):
 
-    BUFFER_SIZE: Final = 2 ** 16
+    BUFFER_SIZE: Final = 2**16
 
-    def __init__(self, name: str, timeout: Optional[float] = None) -> None:
-        if sys.platform == 'win32':
-            name = r'\\.\pipe\{}-{}.pipe'.format(
-                name, base64.urlsafe_b64encode(os.urandom(6)).decode())
+    def __init__(self, name: str, timeout: float | None = None) -> None:
+        if sys.platform == "win32":
+            name = r"\\.\pipe\{}-{}.pipe".format(
+                name, base64.urlsafe_b64encode(os.urandom(6)).decode()
+            )
         else:
-            name = f'{name}.sock'
+            name = f"{name}.sock"
         super().__init__(name, timeout)
-        if sys.platform == 'win32':
-            self.connection = _winapi.CreateNamedPipe(self.name,
+        if sys.platform == "win32":
+            self.connection = _winapi.CreateNamedPipe(
+                self.name,
                 _winapi.PIPE_ACCESS_DUPLEX
                 | _winapi.FILE_FLAG_FIRST_PIPE_INSTANCE
                 | _winapi.FILE_FLAG_OVERLAPPED,
@@ -195,10 +198,10 @@ class IPCServer(IPCBase):
                 self.BUFFER_SIZE,
                 _winapi.NMPWAIT_WAIT_FOREVER,
                 0,  # Use default security descriptor
-                                                      )
+            )
             if self.connection == -1:  # INVALID_HANDLE_VALUE
                 err = _winapi.GetLastError()
-                raise IPCException(f'Invalid handle to pipe: {err}')
+                raise IPCException(f"Invalid handle to pipe: {err}")
         else:
             self.sock_directory = tempfile.mkdtemp()
             sockfile = os.path.join(self.sock_directory, self.name)
@@ -208,8 +211,8 @@ class IPCServer(IPCBase):
             if timeout is not None:
                 self.sock.settimeout(timeout)
 
-    def __enter__(self) -> 'IPCServer':
-        if sys.platform == 'win32':
+    def __enter__(self) -> IPCServer:
+        if sys.platform == "win32":
             # NOTE: It is theoretically possible that this will hang forever if the
             # client never connects, though this can be "solved" by killing the server
             try:
@@ -235,34 +238,36 @@ class IPCServer(IPCBase):
             try:
                 self.connection, _ = self.sock.accept()
             except socket.timeout as e:
-                raise IPCException('The socket timed out') from e
+                raise IPCException("The socket timed out") from e
         return self
 
-    def __exit__(self,
-                 exc_ty: 'Optional[Type[BaseException]]' = None,
-                 exc_val: Optional[BaseException] = None,
-                 exc_tb: Optional[TracebackType] = None,
-                 ) -> None:
-        if sys.platform == 'win32':
+    def __exit__(
+        self,
+        exc_ty: type[BaseException] | None = None,
+        exc_val: BaseException | None = None,
+        exc_tb: TracebackType | None = None,
+    ) -> None:
+        if sys.platform == "win32":
             try:
                 # Wait for the client to finish reading the last write before disconnecting
                 if not FlushFileBuffers(self.connection):
-                    raise IPCException("Failed to flush NamedPipe buffer,"
-                                       "maybe the client hung up?")
+                    raise IPCException(
+                        "Failed to flush NamedPipe buffer," "maybe the client hung up?"
+                    )
             finally:
                 DisconnectNamedPipe(self.connection)
         else:
             self.close()
 
     def cleanup(self) -> None:
-        if sys.platform == 'win32':
+        if sys.platform == "win32":
             self.close()
         else:
             shutil.rmtree(self.sock_directory)
 
     @property
     def connection_name(self) -> str:
-        if sys.platform == 'win32':
+        if sys.platform == "win32":
             return self.name
         else:
             return self.sock.getsockname()
