@@ -6,7 +6,9 @@ ultimately consumed by messages.MessageBuilder.fail(). For more non-trivial mess
 add a method to MessageBuilder and call this instead.
 """
 
-from typing import NamedTuple, Optional
+from __future__ import annotations
+
+from typing import NamedTuple
 from typing_extensions import Final
 
 from mypy import errorcodes as codes
@@ -14,9 +16,9 @@ from mypy import errorcodes as codes
 
 class ErrorMessage(NamedTuple):
     value: str
-    code: Optional[codes.ErrorCode] = None
+    code: codes.ErrorCode | None = None
 
-    def format(self, *args: object, **kwargs: object) -> "ErrorMessage":
+    def format(self, *args: object, **kwargs: object) -> ErrorMessage:
         return ErrorMessage(self.value.format(*args, **kwargs), code=self.code)
 
 
@@ -44,10 +46,6 @@ INVALID_RETURN_TYPE_FOR_ASYNC_GENERATOR: Final = ErrorMessage(
     'The return type of an async generator function should be "AsyncGenerator" or one of its '
     "supertypes"
 )
-INVALID_GENERATOR_RETURN_ITEM_TYPE: Final = ErrorMessage(
-    "The return type of a generator function must be None in"
-    " its third type parameter in Python 2"
-)
 YIELD_VALUE_EXPECTED: Final = ErrorMessage("Yield value expected")
 INCOMPATIBLE_TYPES: Final = "Incompatible types"
 INCOMPATIBLE_TYPES_IN_ASSIGNMENT: Final = "Incompatible types in assignment"
@@ -60,17 +58,23 @@ INCOMPATIBLE_TYPES_IN_ASYNC_WITH_AEXIT: Final = (
     'Incompatible types in "async with" for "__aexit__"'
 )
 INCOMPATIBLE_TYPES_IN_ASYNC_FOR: Final = 'Incompatible types in "async for"'
+INVALID_TYPE_FOR_SLOTS: Final = 'Invalid type for "__slots__"'
+
+ASYNC_FOR_OUTSIDE_COROUTINE: Final = '"async for" outside async function'
+ASYNC_WITH_OUTSIDE_COROUTINE: Final = '"async with" outside async function'
 
 INCOMPATIBLE_TYPES_IN_YIELD: Final = ErrorMessage('Incompatible types in "yield"')
 INCOMPATIBLE_TYPES_IN_YIELD_FROM: Final = ErrorMessage('Incompatible types in "yield from"')
 INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION: Final = "Incompatible types in string interpolation"
-INCOMPATIBLE_TYPES_IN_CAPTURE: Final = ErrorMessage('Incompatible types in capture pattern')
+INCOMPATIBLE_TYPES_IN_CAPTURE: Final = ErrorMessage("Incompatible types in capture pattern")
 MUST_HAVE_NONE_RETURN_TYPE: Final = ErrorMessage('The return type of "{}" must be None')
-INVALID_TUPLE_INDEX_TYPE: Final = ErrorMessage("Invalid tuple index type")
 TUPLE_INDEX_OUT_OF_RANGE: Final = ErrorMessage("Tuple index out of range")
 INVALID_SLICE_INDEX: Final = ErrorMessage("Slice index must be an integer or None")
 CANNOT_INFER_LAMBDA_TYPE: Final = ErrorMessage("Cannot infer type of lambda")
-CANNOT_ACCESS_INIT: Final = 'Cannot access "__init__" directly'
+CANNOT_ACCESS_INIT: Final = (
+    'Accessing "__init__" on an instance is unsound, since instance.__init__ could be from'
+    " an incompatible subclass"
+)
 NON_INSTANCE_NEW_TYPE: Final = ErrorMessage('"__new__" must return a class instance (got {})')
 INVALID_NEW_TYPE: Final = ErrorMessage('Incompatible return type for "__new__"')
 BAD_CONSTRUCTOR_TYPE: Final = ErrorMessage("Unsupported decorated constructor type")
@@ -134,13 +138,10 @@ TYPE_ALWAYS_TRUE_UNIONTYPE: Final = ErrorMessage(
     code=codes.TRUTHY_BOOL,
 )
 FUNCTION_ALWAYS_TRUE: Final = ErrorMessage(
-    'Function {} could always be true in boolean context',
-    code=codes.TRUTHY_BOOL,
-)
-NOT_CALLABLE: Final = '{} not callable'
-PYTHON2_PRINT_FILE_TYPE: Final = (
-    'Argument "file" to "print" has incompatible type "{}"; expected "{}"'
+    "Function {} could always be true in boolean context", code=codes.TRUTHY_BOOL
 )
+NOT_CALLABLE: Final = "{} not callable"
+TYPE_MUST_BE_USED: Final = "Value of type {} must be used"
 
 # Generic
 GENERIC_INSTANCE_VAR_CLASS_ACCESS: Final = (
@@ -151,6 +152,7 @@ BARE_GENERIC: Final = "Missing type parameters for generic type {}"
 IMPLICIT_GENERIC_ANY_BUILTIN: Final = (
     'Implicit generic "Any". Use "{}" and specify generic parameters'
 )
+INVALID_UNPACK = "{} cannot be unpacked (must be tuple or TypeVarTuple)"
 
 # TypeVar
 INCOMPATIBLE_TYPEVAR_VALUE: Final = 'Value of type variable "{}" of {} cannot be {}'
@@ -161,10 +163,13 @@ INVALID_TYPEVAR_ARG_VALUE: Final = 'Invalid type argument value for "{}"'
 TYPEVAR_VARIANCE_DEF: Final = 'TypeVar "{}" may only be a literal bool'
 TYPEVAR_BOUND_MUST_BE_TYPE: Final = 'TypeVar "bound" must be a type'
 TYPEVAR_UNEXPECTED_ARGUMENT: Final = 'Unexpected argument to "TypeVar()"'
+UNBOUND_TYPEVAR: Final = (
+    "A function returning TypeVar should receive at least "
+    "one argument containing the same Typevar"
+)
 
 # Super
 TOO_MANY_ARGS_FOR_SUPER: Final = ErrorMessage('Too many arguments for "super"')
-TOO_FEW_ARGS_FOR_SUPER: Final = ErrorMessage('Too few arguments for "super"', codes.CALL_ARG)
 SUPER_WITH_SINGLE_ARG_NOT_SUPPORTED: Final = ErrorMessage(
     '"super" with a single argument not supported'
 )
@@ -190,9 +195,6 @@ MISSING_OR_INVALID_SELF_TYPE: Final = ErrorMessage(
 ERASED_SELF_TYPE_NOT_SUPERTYPE: Final = ErrorMessage(
     'The erased type of self "{}" is not a supertype of its class "{}"'
 )
-INVALID_SELF_TYPE_OR_EXTRA_ARG: Final = ErrorMessage(
-    "Invalid type for self, or extra argument type in function annotation"
-)
 
 # Final
 CANNOT_INHERIT_FROM_FINAL: Final = ErrorMessage('Cannot inherit from final class "{}"')
@@ -206,7 +208,7 @@ CANNOT_MAKE_DELETABLE_FINAL: Final = ErrorMessage("Deletable attribute cannot be
 
 # Enum
 ENUM_MEMBERS_ATTR_WILL_BE_OVERRIDEN: Final = ErrorMessage(
-    'Assigned "__members__" will be overriden by "Enum" internally'
+    'Assigned "__members__" will be overridden by "Enum" internally'
 )
 
 # ClassVar
@@ -218,10 +220,8 @@ CANNOT_OVERRIDE_CLASS_VAR: Final = ErrorMessage(
     'Cannot override class variable (previously declared on base class "{}") with instance '
     "variable"
 )
-CLASS_VAR_WITH_TYPEVARS: Final = 'ClassVar cannot contain type variables'
-CLASS_VAR_OUTSIDE_OF_CLASS: Final = (
-    'ClassVar can only be used for assignments in class body'
-)
+CLASS_VAR_WITH_TYPEVARS: Final = "ClassVar cannot contain type variables"
+CLASS_VAR_OUTSIDE_OF_CLASS: Final = "ClassVar can only be used for assignments in class body"
 
 # Protocol
 RUNTIME_PROTOCOL_EXPECTED: Final = ErrorMessage(
