@@ -3,47 +3,40 @@
 These happen after semantic analysis and before type checking.
 """
 
-from typing import List, Set, Optional
+from __future__ import annotations
+
 from typing_extensions import Final
 
-from mypy.nodes import (
-    Node, TypeInfo, Var, Decorator, OverloadedFuncDef, SymbolTable, CallExpr, PromoteExpr,
-)
-from mypy.types import Instance, Type
 from mypy.errors import Errors
+from mypy.nodes import (
+    IMPLICITLY_ABSTRACT,
+    IS_ABSTRACT,
+    CallExpr,
+    Decorator,
+    FuncDef,
+    Node,
+    OverloadedFuncDef,
+    PromoteExpr,
+    SymbolTable,
+    TypeInfo,
+    Var,
+)
 from mypy.options import Options
+from mypy.types import Instance, Type
 
 # Hard coded type promotions (shared between all Python versions).
 # These add extra ad-hoc edges to the subtyping relation. For example,
 # int is considered a subtype of float, even though there is no
 # subclass relationship.
-TYPE_PROMOTIONS: Final = {
-    'builtins.int': 'float',
-    'builtins.float': 'complex',
-}
-
-# Hard coded type promotions for Python 3.
-#
 # Note that the bytearray -> bytes promotion is a little unsafe
 # as some functions only accept bytes objects. Here convenience
 # trumps safety.
-TYPE_PROMOTIONS_PYTHON3: Final = TYPE_PROMOTIONS.copy()
-TYPE_PROMOTIONS_PYTHON3.update({
-    'builtins.bytearray': 'bytes',
-    'builtins.memoryview': 'bytes',
-})
-
-# Hard coded type promotions for Python 2.
-#
-# These promotions are unsafe, but we are doing them anyway
-# for convenience and also for Python 3 compatibility
-# (bytearray -> str).
-TYPE_PROMOTIONS_PYTHON2: Final = TYPE_PROMOTIONS.copy()
-TYPE_PROMOTIONS_PYTHON2.update({
-    'builtins.str': 'unicode',
-    'builtins.bytearray': 'str',
-    'builtins.memoryview': 'str',
-})
+TYPE_PROMOTIONS: Final = {
+    "builtins.int": "float",
+    "builtins.float": "complex",
+    "builtins.bytearray": "bytes",
+    "builtins.memoryview": "bytes",
+}
 
 
 def calculate_class_abstract_status(typ: TypeInfo, is_stub_file: bool, errors: Errors) -> None:
@@ -55,9 +48,10 @@ def calculate_class_abstract_status(typ: TypeInfo, is_stub_file: bool, errors: E
     """
     if typ.typeddict_type:
         return  # TypedDict can't be abstract
-    concrete: Set[str] = set()
-    abstract: List[str] = []
-    abstract_in_this_class: List[str] = []
+    concrete: set[str] = set()
+    # List of abstract attributes together with their abstract status
+    abstract: list[tuple[str, int]] = []
+    abstract_in_this_class: list[str] = []
     if typ.is_newtype:
         # Special case: NewTypes are considered as always non-abstract, so they can be used as:
         #     Config = NewType('Config', Mapping[str, str])
@@ -73,22 +67,26 @@ def calculate_class_abstract_status(typ: TypeInfo, is_stub_file: bool, errors: E
                 # different items have a different abstract status, there
                 # should be an error reported elsewhere.
                 if node.items:  # can be empty for invalid overloads
-                    func: Optional[Node] = node.items[0]
+                    func: Node | None = node.items[0]
                 else:
                     func = None
             else:
                 func = node
             if isinstance(func, Decorator):
-                fdef = func.func
-                if fdef.is_abstract and name not in concrete:
+                func = func.func
+            if isinstance(func, FuncDef):
+                if (
+                    func.abstract_status in (IS_ABSTRACT, IMPLICITLY_ABSTRACT)
+                    and name not in concrete
+                ):
                     typ.is_abstract = True
-                    abstract.append(name)
+                    abstract.append((name, func.abstract_status))
                     if base is typ:
                         abstract_in_this_class.append(name)
             elif isinstance(node, Var):
                 if node.is_abstract_var and name not in concrete:
                     typ.is_abstract = True
-                    abstract.append(name)
+                    abstract.append((name, IS_ABSTRACT))
                     if base is typ:
                         abstract_in_this_class.append(name)
             concrete.add(name)
@@ -97,32 +95,37 @@ def calculate_class_abstract_status(typ: TypeInfo, is_stub_file: bool, errors: E
     # implement some methods.
     typ.abstract_attributes = sorted(abstract)
     if is_stub_file:
-        if typ.declared_metaclass and typ.declared_metaclass.type.fullname == 'abc.ABCMeta':
+        if typ.declared_metaclass and typ.declared_metaclass.type.fullname == "abc.ABCMeta":
             return
         if typ.is_protocol:
             return
         if abstract and not abstract_in_this_class:
+
             def report(message: str, severity: str) -> None:
                 errors.report(typ.line, typ.column, message, severity=severity)
 
-            attrs = ", ".join('"{}"'.format(attr) for attr in sorted(abstract))
-            report("Class {} has abstract attributes {}".format(typ.fullname, attrs), 'error')
-            report("If it is meant to be abstract, add 'abc.ABCMeta' as an explicit metaclass",
-                   'note')
+            attrs = ", ".join(f'"{attr}"' for attr, _ in sorted(abstract))
+            report(f"Class {typ.fullname} has abstract attributes {attrs}", "error")
+            report(
+                "If it is meant to be abstract, add 'abc.ABCMeta' as an explicit metaclass", "note"
+            )
     if typ.is_final and abstract:
-        attrs = ", ".join('"{}"'.format(attr) for attr in sorted(abstract))
-        errors.report(typ.line, typ.column,
-                      "Final class {} has abstract attributes {}".format(typ.fullname, attrs))
+        attrs = ", ".join(f'"{attr}"' for attr, _ in sorted(abstract))
+        errors.report(
+            typ.line, typ.column, f"Final class {typ.fullname} has abstract attributes {attrs}"
+        )
 
 
 def check_protocol_status(info: TypeInfo, errors: Errors) -> None:
     """Check that all classes in MRO of a protocol are protocols"""
     if info.is_protocol:
         for type in info.bases:
-            if not type.type.is_protocol and type.type.fullname != 'builtins.object':
+            if not type.type.is_protocol and type.type.fullname != "builtins.object":
+
                 def report(message: str, severity: str) -> None:
                     errors.report(info.line, info.column, message, severity=severity)
-                report('All bases of a protocol must be protocols', 'error')
+
+                report("All bases of a protocol must be protocols", "error")
 
 
 def calculate_class_vars(info: TypeInfo) -> None:
@@ -140,33 +143,40 @@ def calculate_class_vars(info: TypeInfo) -> None:
         if isinstance(node, Var) and node.info and node.is_inferred and not node.is_classvar:
             for base in info.mro[1:]:
                 member = base.names.get(name)
-                if (member is not None
-                        and isinstance(member.node, Var)
-                        and member.node.is_classvar):
+                if member is not None and isinstance(member.node, Var) and member.node.is_classvar:
                     node.is_classvar = True
 
 
-def add_type_promotion(info: TypeInfo, module_names: SymbolTable, options: Options) -> None:
+def add_type_promotion(
+    info: TypeInfo, module_names: SymbolTable, options: Options, builtin_names: SymbolTable
+) -> None:
     """Setup extra, ad-hoc subtyping relationships between classes (promotion).
 
     This includes things like 'int' being compatible with 'float'.
     """
     defn = info.defn
-    promote_target: Optional[Type] = None
+    promote_targets: list[Type] = []
     for decorator in defn.decorators:
         if isinstance(decorator, CallExpr):
             analyzed = decorator.analyzed
             if isinstance(analyzed, PromoteExpr):
                 # _promote class decorator (undocumented feature).
-                promote_target = analyzed.type
-    if not promote_target:
-        promotions = (TYPE_PROMOTIONS_PYTHON3 if options.python_version[0] >= 3
-                      else TYPE_PROMOTIONS_PYTHON2)
-        if defn.fullname in promotions:
-            target_sym = module_names.get(promotions[defn.fullname])
+                promote_targets.append(analyzed.type)
+    if not promote_targets:
+        if defn.fullname in TYPE_PROMOTIONS:
+            target_sym = module_names.get(TYPE_PROMOTIONS[defn.fullname])
             # With test stubs, the target may not exist.
             if target_sym:
                 target_info = target_sym.node
                 assert isinstance(target_info, TypeInfo)
-                promote_target = Instance(target_info, [])
-    defn.info._promote = promote_target
+                promote_targets.append(Instance(target_info, []))
+    # Special case the promotions between 'int' and native integer types.
+    # These have promotions going both ways, such as from 'int' to 'i64'
+    # and 'i64' to 'int', for convenience.
+    if defn.fullname == "mypy_extensions.i64" or defn.fullname == "mypy_extensions.i32":
+        int_sym = builtin_names["int"]
+        assert isinstance(int_sym.node, TypeInfo)
+        int_sym.node._promote.append(Instance(defn.info, []))
+        defn.info.alt_promote = int_sym.node
+    if promote_targets:
+        defn.info._promote.extend(promote_targets)
