@@ -1,22 +1,25 @@
 #!/usr/bin/env python3
+
+from __future__ import annotations
+
 import subprocess
 from subprocess import Popen
-from sys import argv, exit, executable
+from sys import argv, executable, exit
 
 # Slow test suites
-CMDLINE = 'PythonCmdline'
-SAMPLES = 'SamplesSuite'
-TYPESHED = 'TypeshedSuite'
-PEP561 = 'PEP561Suite'
-EVALUATION = 'PythonEvaluation'
-DAEMON = 'testdaemon'
-STUBGEN_CMD = 'StubgenCmdLine'
-STUBGEN_PY = 'StubgenPythonSuite'
-MYPYC_RUN = 'TestRun'
-MYPYC_RUN_MULTI = 'TestRunMultiFile'
-MYPYC_EXTERNAL = 'TestExternal'
-MYPYC_COMMAND_LINE = 'TestCommandLine'
-ERROR_STREAM = 'ErrorStreamSuite'
+CMDLINE = "PythonCmdline"
+SAMPLES = "SamplesSuite"
+TYPESHED = "TypeshedSuite"
+PEP561 = "PEP561Suite"
+EVALUATION = "PythonEvaluation"
+DAEMON = "testdaemon"
+STUBGEN_CMD = "StubgenCmdLine"
+STUBGEN_PY = "StubgenPythonSuite"
+MYPYC_RUN = "TestRun"
+MYPYC_RUN_MULTI = "TestRunMultiFile"
+MYPYC_EXTERNAL = "TestExternal"
+MYPYC_COMMAND_LINE = "TestCommandLine"
+ERROR_STREAM = "ErrorStreamSuite"
 
 
 ALL_NON_FAST = [
@@ -49,43 +52,40 @@ MYPYC_OPT_IN = [MYPYC_RUN, MYPYC_RUN_MULTI]
 # time to run.
 cmds = {
     # Self type check
-    'self': [executable, '-m', 'mypy', '--config-file', 'mypy_self_check.ini', '-p', 'mypy'],
+    "self": [executable, "-m", "mypy", "--config-file", "mypy_self_check.ini", "-p", "mypy"],
     # Lint
-    'lint': ['flake8', '-j0'],
+    "lint": ["flake8", "-j0"],
+    "format-black": ["black", "."],
+    "format-isort": ["isort", "."],
     # Fast test cases only (this is the bulk of the test suite)
-    'pytest-fast': ['pytest', '-q', '-k', 'not (%s)' % ' or '.join(ALL_NON_FAST)],
+    "pytest-fast": ["pytest", "-q", "-k", f"not ({' or '.join(ALL_NON_FAST)})"],
     # Test cases that invoke mypy (with small inputs)
-    'pytest-cmdline': ['pytest', '-q', '-k', ' or '.join([CMDLINE,
-                                                          EVALUATION,
-                                                          STUBGEN_CMD,
-                                                          STUBGEN_PY])],
+    "pytest-cmdline": [
+        "pytest",
+        "-q",
+        "-k",
+        " or ".join([CMDLINE, EVALUATION, STUBGEN_CMD, STUBGEN_PY]),
+    ],
     # Test cases that may take seconds to run each
-    'pytest-slow': ['pytest', '-q', '-k', ' or '.join(
-        [SAMPLES,
-         TYPESHED,
-         DAEMON,
-         MYPYC_EXTERNAL,
-         MYPYC_COMMAND_LINE,
-         ERROR_STREAM])],
-
+    "pytest-slow": [
+        "pytest",
+        "-q",
+        "-k",
+        " or ".join([SAMPLES, TYPESHED, DAEMON, MYPYC_EXTERNAL, MYPYC_COMMAND_LINE, ERROR_STREAM]),
+    ],
     # Test cases that might take minutes to run
-    'pytest-extra': ['pytest', '-q', '-k', ' or '.join(PYTEST_OPT_IN)],
-
+    "pytest-extra": ["pytest", "-q", "-k", " or ".join(PYTEST_OPT_IN)],
     # Test cases to run in typeshed CI
-    'typeshed-ci': ['pytest', '-q', '-k', ' or '.join([CMDLINE,
-                                                       EVALUATION,
-                                                       SAMPLES,
-                                                       TYPESHED])],
-
+    "typeshed-ci": ["pytest", "-q", "-k", " or ".join([CMDLINE, EVALUATION, SAMPLES, TYPESHED])],
     # Mypyc tests that aren't run by default, since they are slow and rarely
     # fail for commits that don't touch mypyc
-    'mypyc-extra': ['pytest', '-q', '-k', ' or '.join(MYPYC_OPT_IN)],
+    "mypyc-extra": ["pytest", "-q", "-k", " or ".join(MYPYC_OPT_IN)],
 }
 
 # Stop run immediately if these commands fail
-FAST_FAIL = ['self', 'lint']
+FAST_FAIL = ["self", "lint"]
 
-EXTRA_COMMANDS = ('pytest-extra', 'mypyc-extra', 'typeshed-ci')
+EXTRA_COMMANDS = ("pytest-extra", "mypyc-extra", "typeshed-ci")
 DEFAULT_COMMANDS = [cmd for cmd in cmds if cmd not in EXTRA_COMMANDS]
 
 assert all(cmd in cmds for cmd in FAST_FAIL)
@@ -94,10 +94,10 @@ assert all(cmd in cmds for cmd in FAST_FAIL)
 def run_cmd(name: str) -> int:
     status = 0
     cmd = cmds[name]
-    print(f'run {name}: {cmd}')
+    print(f"run {name}: {cmd}")
     proc = subprocess.run(cmd, stderr=subprocess.STDOUT)
     if proc.returncode:
-        print('\nFAILED: %s' % name)
+        print("\nFAILED: %s" % name)
         status = proc.returncode
         if name in FAST_FAIL:
             exit(status)
@@ -106,19 +106,17 @@ def run_cmd(name: str) -> int:
 
 def start_background_cmd(name: str) -> Popen:
     cmd = cmds[name]
-    proc = subprocess.Popen(cmd,
-                            stderr=subprocess.STDOUT,
-                            stdout=subprocess.PIPE)
+    proc = subprocess.Popen(cmd, stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
     return proc
 
 
 def wait_background_cmd(name: str, proc: Popen) -> int:
     output = proc.communicate()[0]
     status = proc.returncode
-    print(f'run {name}: {cmds[name]}')
+    print(f"run {name}: {cmds[name]}")
     if status:
         print(output.decode().rstrip())
-        print('\nFAILED: %s' % name)
+        print("\nFAILED:", name)
         if name in FAST_FAIL:
             exit(status)
     return status
@@ -128,10 +126,12 @@ def main() -> None:
     prog, *args = argv
 
     if not set(args).issubset(cmds):
-        print("usage:", prog, " ".join('[%s]' % k for k in cmds))
+        print("usage:", prog, " ".join(f"[{k}]" for k in cmds))
         print()
-        print('Run the given tests. If given no arguments, run everything except'
-              + ' pytest-extra and mypyc-extra.')
+        print(
+            "Run the given tests. If given no arguments, run everything except"
+            + " pytest-extra and mypyc-extra."
+        )
         exit(1)
 
     if not args:
@@ -139,16 +139,16 @@ def main() -> None:
 
     status = 0
 
-    if 'self' in args and 'lint' in args:
+    if "self" in args and "lint" in args:
         # Perform lint and self check in parallel as it's faster.
-        proc = start_background_cmd('lint')
-        cmd_status = run_cmd('self')
+        proc = start_background_cmd("lint")
+        cmd_status = run_cmd("self")
         if cmd_status:
             status = cmd_status
-        cmd_status = wait_background_cmd('lint', proc)
+        cmd_status = wait_background_cmd("lint", proc)
         if cmd_status:
             status = cmd_status
-        args = [arg for arg in args if arg not in ('self', 'lint')]
+        args = [arg for arg in args if arg not in ("self", "lint")]
 
     for arg in args:
         cmd_status = run_cmd(arg)
@@ -158,5 +158,5 @@ def main() -> None:
     exit(status)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
